/**************************************************************************//**
*   @file   ad5553.c
*   @brief  Implementation of AD5553 Driver for NIOS processor.
*   @author grohmast
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 158
******************************************************************************/
#define AD5553_GLOBALS

/*****************************************************************************/
/* Include Files															 */
/*****************************************************************************/
#include "ad5553.h"
#include <altera_avalon_pio_regs.h>         // PIO access macros
#include <altera_avalon_spi.h>              // SPI
#include <unistd.h>                         // usleep()
#include <system.h>
#if (USE_AD5553 == 1)

/*****************************************************************************/
/*  Private variables                                                        */
/*****************************************************************************/
unsigned char	txBuffer[4] = {0,};			// Tx SPI Buffer.
unsigned char   rxBuffer[4] = {0,};			// Rx SPI Buffer.

/**************************************************************************//**
 * @brief Initialize the ad5553 board and enable SPI.
 *
 * @param  None.
 *
 * @return SUCCESS.
******************************************************************************/
int AD5553_Init(void)
{
	dacValue = 0;
	SetCtrl(AD5553_SPI_CFG); 	// enable output buffers of SPORT SPI interface

    return(SUCCESS);
}

/**************************************************************************//**
 * @brief Set data
 *
 * @param value - The data
 *
 * @return SUCCESS.
******************************************************************************/
int AD5553_Set(int value)
{
	txBuffer[0] = (unsigned char) ((value & 0x00ff00) >> 8);
	txBuffer[1] = (unsigned char)  (value & 0x0000ff);
    alt_avalon_spi_command(AD5553_SPI, 	// base address of SPI module
    					   0,         	// slave number
    					   2,           // write_length
    					   txBuffer,    // *write_data
    					   0,           // read_length
    					   txBuffer,    // *read_data
    					   0);          // flags

    return(SUCCESS);
}

/**************************************************************************//**
 * @brief Update the device's monitored and controlled parameters
 *
 * @param None
 *
 * @return Always returns success
******************************************************************************/
int AD5553_UpdateParameters(void)
{
	usleep(50000);
	outputVoltage = (float)dacValue * (-5.0f/16383);
    AD5553_Set(dacValue);

    return(SUCCESS);
}
#endif /* USE_ad5553 */
